Segment Intelligente Analyseverfahren, SW-Einheit Straßensubsegmentanalyse
==========================================================================

In der Straßensubsegmentanalyse wird unter Einsatz von Fuzzy-Methoden ein
linguistischer Messquerschnitts- und Straßenteilsegmentzustand abgeleitet. Dazu
werden aus den Mess- und Modellgrößen verschiedene Zustände bestimmt (z.B. eine
Reisegeschwindigkeit _mittel_ oder eine lokale Verkehrslage _Stau_). Der
Benutzer kann im späteren Einsatz die Zugehörigkeitsfunktionen von Verkehrs- und
Umfelddaten ändern. Ausgaben der Straßensubsegmentanalyse sind u.a.
fuzzyfizierte lokale Verkehrslagen z.B. für eine grafische Darstellung oder als
Eingang für die Umfassende Datenanalyse.


Installation
------------

Die ZIP-Datei entpacken und das enthaltene Verzeichnis neben die anderen
SW-Einheiten ins Installationsverzeichnis kopieren.

Die folgenden Abhängigkeiten müssen ebenfalls im Installationsverzeichnis
vorhanden sein:

*   `de.bsvrz.dav.daf` ab Version 3.9.7
*   `de.bsvrz.sys.funclib.application` ab Version 3.9.7
*   `de.bsvrz.iav.fuzzylib', version` ab Version 3.0.0


Verwendung
----------

Die Applikation Straßensubsegmentanalyse wird wie folgt gestartet:

    java -jar de.bsvrz.iav.subsegana.jar [Argumente]

Beispiel

    java -jar de.bsvrz.iav.subsegana.jar -datenverteiler=localhost:8083 -benutzer=Tester -authentifizierung=passwd

Neben den Argumenten der Datenverteilerapplikationsfunktionen werden
unterstützt:

*   `-objekte=` Eine kommagetrennte Liste von PIDs von zu verwaltenden Objekten.
    Die PIDs können Systemobjekten, Konfigurationsbereichen oder
    Systemobjekttypen sein. Es werden alle Objekte aus den
    Konfigurationsbereichen bzw. alle Objekte der Systemobjekttypen verwaltet.
    Unterstützte Objekte sind Messquerschnitte, Straßenteilsegmente,
    Umfelddatenmessstellen, Umfelddatensensoren für Helligkeit,
    Niederschlagsintensität, Sichtweite, Windgeschwindigkeit und Windrichtung.
    Wird der Parameter nicht angegeben, werden per Default alle in der
    Konfiguration versorgten unterstützten Objekte verwaltet.
*   `-maxZyklen=2` Die maximale Anzahl von zwischengespeicherten
    Datensatzzyklen. Nur solche zwischengespeicherten Zyklen können in Regeln
    verwendet werden. Der Defaultwert sind zwei Zyklen. Der Wert muss zwischen
    eins und neun liegen.

Um Messquerschnitte und Straßenteilsegmente gemäß den Anforderungen abhängig von
Lage und Anzahl der Fahrstreifen zu parametrieren kann der
Standardparametersetzer verwendet werden. Gestartet wird er mit:

    java -classpath de.bsvrz.iav.subsegana.jar de.bsvrz.iav.subsegana.subsegana.application.Standardparametersetzer [Argumente]

Beispiel

    java -classpath de.bsvrz.iav.subsegana.jar de.bsvrz.iav.subsegana.subsegana.application.Standardparametersetzer -datenverteiler=localhost:8083 -benutzer=Tester -authentifizierung=passwd

Die verwaltenden Objekte und deren Beziehungen untereinander sind in der
Konfiguration versorgt:

*   `typ.messQuerschnittAllgemein`
    *   `atg.punktLiegtAufLinienObjekt`
*   `typ.messQuerschnitt`
    * Menge `FahrStreifen`
*   `typ.straßenSegment`
    *   `atg.straßenSegment`
    *   `atg.linie`
    *   `atg.bestehtAusLinienObjekten`
*   `typ.straßenTeilSegment`
    *   `atg.straßenTeilSegment`
    *   `atg.linie`
*   `typ.umfeldDatenMessStelle`
    *   Menge `UmfeldDatenSensoren`

Die Fuzzy-Sets zur Fuzzyfizierung von Messwerten werden als Onlineparameter
versorgt, ebenso wie die Regelbasen zur Ableitung von weiteren Fuzzy-Werten:

*   `typ.messQuerschnittAllgemein` und `typ.straßenTeilSegment`
    *   `atg.anteilLkwFuzzy`
    *   `atg.bemessungsDichteFuzzy`
    *   `atg.verkehrGeschwindigkeitFuzzy`
    *   `atg.verkehrsStärkeStundeFuzzy`
    *   `atg.verkehrsUnruheFuzzy`
    *   `atg.fuzzyRegelbasis`
*   `typ.umfeldDatenMessStelle`
    *   `atg.ufdmsParameterGlätteFuzzy`
    *   `atg.fuzzyRegelbasis`
*   `typ.ufdsHelligkeit`
    *   `atg.ufdsParameterHelligkeitFuzzy`
*   `typ.ufdsNiederschlagsIntensität`
    *   `atg.ufdsParameterNiederschlagsIntensitätFuzzy`
*   `typ.ufdsSichtWeite`
    *   `atg.ufdsParameterSichtWeiteFuzzy`
*   `typ.ufdsWindGeschwindigkeitMittelWert`
    *   `atg.ufdsParameterWindGeschwindigkeitMittelWertFuzzy`
*   `typ.ufdsWindRichtung`
    *   `atg.ufdsParameterWindRichtungFuzzy`

Die folgenden Messwerte werden fuzzyfiziert:

*   `typ.messQuerschnittAllgemein`
    *   `atg.verkehrsDatenKurzZeitMq`, `asp.analyse`
*   `typ.umfeldDatenMessStelle`
    *    `atg.ufdmsGlätte`, `asp.prognose`
*   `typ.ufdsHelligkeit`
    *    `atg.ufdsHelligkeit`, `asp.messWertErsetzung`
*   `typ.ufdsNiederschlagsIntensität`
    *    `atg.ufdsNiederschlagsIntensität`, `asp.messWertErsetzung`
*   `typ.ufdsSichtWeite`
    *    `atg.ufdsSichtWeite`, `asp.messWertErsetzung`
*   `typ.ufdsWindGeschwindigkeitMittelWert`
    *    `atg.ufdsWindGeschwindigkeitMittelWert`, `asp.messWertErsetzung`
*   `typ.ufdsWindRichtung`
    *    `atg.ufdsWindRichtung`, `asp.messWertErsetzung`

Die folgenden Fuzzy-Werte werden bestimmt:

*   `typ.messQuerschnittAllgemein` und `typ.straßenTeilSegment`
    *   `atg.anteilLkw`, `asp.fuzzy`
    *   `atg.bemessungsDichte`, `asp.fuzzy`
    *   `atg.verkehrGeschwindigkeit`, `asp.fuzzy`
    *   `atg.verkehrsStärkeStunde`, `asp.fuzzy`
    *   `atg.verkehrsUnruhe`, `asp.fuzzy`
    *   `atg.verkehrsZustand`, `asp.fuzzy`
    *   `atg.verkehrsZustandUnruhe`, `asp.fuzzy`
*   `typ.umfeldDatenMessStelle`
    *   `atg.ufdmsGlätteFuzzy`, `asp.fuzzy`
    *   `atg.ufdmsSichtVerhältnisseFuzzy`, `asp.fuzzy`
    *   `atg.ufdmsFahrBahnZustandFuzzy`, `asp.fuzzy`
*   `typ.ufdsHelligkeit`
    *   `atg.ufdsHelligkeitFuzzy`, `asp.fuzzy`
*   `typ.ufdsNiederschlagsIntensität`
    *   `atg.ufdsNiederschlagsIntensitätFuzzy`, `asp.fuzzy`
*   `typ.ufdsSichtWeite`
    *   `atg.ufdsSichtWeiteFuzzy`, `asp.fuzzy`
*   `typ.ufdsWindGeschwindigkeitMittelWert`
    *   `atg.ufdsWindGeschwindigkeitMittelWertFuzzy`, `asp.fuzzy`
*   `typ.ufdsWindRichtung`
    *   `atg.ufdsWindRichtungFuzzy`, `asp.fuzzy`

Die folgenden Störfallzustände werden bestimmt:

*   `typ.messQuerschnittAllgemein` und `typ.straßenTeilSegment`
    * `atg.störfallZustand`, `asp.störfallVerfahrenFuzzy`


Kontakt
-------

BitCtrl Systems GmbH<br>
Weißenfelser Straße 67<br>
04229 Leipzig<br>
Telefon: +49 341-490670<br>
E-Mail: info@bitctrl.de
