/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain;

import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;

import java.util.*;
import java.util.stream.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.Verkehrsgroessen.*;

/**
 * Aggregiert die Messwerte von Messquerschnitten die zu einem
 * Straßenteilsegment gehören.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Aggregation {

    private final Event<Datensatz<Strassenteilsegment, Collection<Messwert>>> messwertAggregiert = new Event<>();

    private final Map<Messquerschnitt, Datensatz<Messquerschnitt, Collection<Messwert>>> messwerte = new LinkedHashMap<>();

    /**
     * Test-Seam.
     */
    Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> getMesswerte() {
        return new ArrayList<>(messwerte.values());
    }

    public void aggregiereMesswert(Datensatz<Messquerschnitt, Collection<Messwert>> messwerte) {
        messwerte.getSystemobjekt().getStrassenteilsegment()
                .ifPresent(sts -> aggregiereFuerStrassenteilsegment(sts, messwerte));
    }

    private void aggregiereFuerStrassenteilsegment(Strassenteilsegment strassenteilsegment,
                                                   Datensatz<Messquerschnitt, Collection<Messwert>> neuerMesswert) {
        Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> aktuelleMesswerte = bestimmeZwischengespeicherteMesswerte(strassenteilsegment);
        if (istMesswertImGleichenZyklus(aktuelleMesswerte, neuerMesswert)) {
            messwertZwischenspeichern(neuerMesswert);
            aktuelleMesswerte.add(neuerMesswert);

            if (zyklusKomplett(strassenteilsegment, aktuelleMesswerte)) {
                veroeffentlicheAggregiertenMesswert(strassenteilsegment, aktuelleMesswerte);
            }
        } else if (fehltMesswertImZyklus(aktuelleMesswerte, neuerMesswert)) {
            veroeffentlicheAggregiertenMesswert(strassenteilsegment, aktuelleMesswerte);
            messwertZwischenspeichern(neuerMesswert);
        }
    }

    private Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> bestimmeZwischengespeicherteMesswerte(Strassenteilsegment strassenteilsegment) {
        return strassenteilsegment.getMessquerschnitte().stream()
                .map(this.messwerte::get)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
    }

    private static boolean istMesswertImGleichenZyklus(Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> aktuelleMesswerte,
                                                       Datensatz<Messquerschnitt, Collection<Messwert>> neuerDatensatz) {
        if (aktuelleMesswerte.isEmpty()) {
            return true;
        }
        Datensatz<Messquerschnitt, Collection<Messwert>> ersterAktuellerDatensatz = aktuelleMesswerte.iterator().next();
        return ersterAktuellerDatensatz.getZeitstempel().equals(neuerDatensatz.getZeitstempel());
    }

    private static boolean fehltMesswertImZyklus(Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> aktuelleDatensaetze,
                                                 Datensatz<Messquerschnitt, Collection<Messwert>> neuerDatensatz) {
        Datensatz<Messquerschnitt, Collection<Messwert>> ersterAktuellerDatensatz = aktuelleDatensaetze.iterator().next();
        return !ersterAktuellerDatensatz.getZeitstempel().equals(neuerDatensatz.getZeitstempel());
    }

    private void messwertZwischenspeichern(Datensatz<Messquerschnitt, Collection<Messwert>> messwert) {
        messwerte.put(messwert.getSystemobjekt(), messwert);
    }

    private static boolean zyklusKomplett(Strassenteilsegment strassenteilsegment,
                                          Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> aktuelleMesswerte) {
        return aktuelleMesswerte.size() == strassenteilsegment.getMessquerschnitte().size();
    }

    private void veroeffentlicheAggregiertenMesswert(Strassenteilsegment strassenteilsegment,
                                                     Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> messwerte) {
        Collection<Messwert> datum = bestimmeMittelwert(messwerte);
        Datensatz<Messquerschnitt, Collection<Messwert>> ersterDatensatz = messwerte.iterator().next();
        messwertAggregiert.send(new Datensatz<>(ersterDatensatz.getZeitstempel(), strassenteilsegment, datum));
        leereZwischenspeicher(strassenteilsegment);
    }

    private static Collection<Messwert> bestimmeMittelwert(Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> messwerte) {
        Collection<Messwert> aggregierteMesswerte = new ArrayList<>();
        aggregierteMesswerte.add(bestimmeMittelwertFuer(QKFZ, messwerte));
        aggregierteMesswerte.add(bestimmeMittelwertFuer(VKFZ, messwerte));
        aggregierteMesswerte.add(bestimmeMittelwertFuer(SKFZ, messwerte));
        aggregierteMesswerte.add(bestimmeMittelwertFuer(ALKW, messwerte));
        aggregierteMesswerte.add(bestimmeMittelwertFuer(KB, messwerte));
        return aggregierteMesswerte;
    }

    private static Messwert bestimmeMittelwertFuer(String messwertname,
                                                   Collection<Datensatz<Messquerschnitt, Collection<Messwert>>> messwerte) {
        OptionalDouble mittelwert = messwerte.stream()
                .map(Datensatz::getDatum)
                .flatMap(Collection::stream)
                .filter(Messwert::nonNull)
                .filter(m -> m.getName().equals(messwertname))
                .mapToDouble(Messwert::getWert)
                .average();
        if (mittelwert.isPresent()) {
            return new Messwert(messwertname, mittelwert.getAsDouble());
        } else {
            return new Messwert(messwertname);
        }
    }

    private void leereZwischenspeicher(Strassenteilsegment strassenteilsegment) {
        strassenteilsegment.getMessquerschnitte()
                .forEach(messwerte::remove);
    }

    public Event<Datensatz<Strassenteilsegment, Collection<Messwert>>> messwertAggregiert() {
        return messwertAggregiert;
    }

}
