/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.shared;

import java.util.*;

/**
 * Repräsentiert einen benannten Messwert.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class Messwert {

    private final String name;
    private final Double wert;

    public Messwert(String name) {
        this(name, null);
    }

    public Messwert(String name, double wert) {
        this(name, Double.valueOf(wert));
    }

    private Messwert(String name, Double wert) {
        this.name = Objects.requireNonNull(name, "name");
        this.wert = wert;
    }

    public String getName() {
        return name;
    }

    public double getWert() {
        return wert;
    }

    public boolean isNull() {
        return wert == null;
    }

    public boolean nonNull() {
        return !isNull();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Messwert that = (Messwert) o;
        return Objects.equals(name, that.name) &&
                Objects.equals(wert, that.wert);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, wert);
    }

    @Override
    public String toString() {
        return "Messwert{" +
                "name='" + name + '\'' +
                ", wert=" + wert +
                '}';
    }

}
