/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten;

import java.util.*;

/**
 * Repräsentiert eine Länge in Meter.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class Laenge {

    public static final Laenge UNDEFINIERT = new Laenge(-1);

    private final long wert;

    private Laenge(double wert) {
        this.wert = Math.round(wert * 100);
    }

    public static Laenge von(double wert) {
        if (wert < 0) {
            throw new IllegalArgumentException("Länge muss größer oder gleich 0 sein: " + wert);
        }
        return new Laenge(wert);
    }

    public Double getWert() {
        return wert / 100.0;
    }

    public boolean isNull() {
        return wert < 0;
    }

    public boolean nonNull() {
        return !isNull();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Laenge laenge = (Laenge) o;
        return wert == laenge.wert;
    }

    @Override
    public int hashCode() {
        return Objects.hash(wert);
    }

    @Override
    public String toString() {
        if (nonNull()) {
            return getWert() + " m";
        } else {
            if (this == UNDEFINIERT) {
                return "undefiniert";
            } else {
                throw new IllegalArgumentException("unreachable code");
            }
        }
    }

}
