/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;

import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten.Verkehrsgroessen.*;

/**
 * Fasst Fabrikmethoden zusammen zum Erzeugen von Fuzzy-Sets für Verkehrsdaten.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class VerkehrsdatenStandardparameterFactory {

    private static final String VKFZ_1 = "VKfz1";

    private VerkehrsdatenStandardparameterFactory() {
        // utility class
    }

    public static LinguistischeVariable geschwindigkeitSzenario1() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 25, 35));
        fuzzySets.add(FuzzySet.trapez("mittel", 25, 35, 75, 85));
        fuzzySets.add(FuzzySet.trapez("hoch", 75, 85, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable geschwindigkeitSzenario2() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 27, 36));
        fuzzySets.add(FuzzySet.trapez("mittel", 27, 36, 77, 86));
        fuzzySets.add(FuzzySet.trapez("hoch", 75, 86, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable geschwindigkeitSzenario3() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 28, 36));
        fuzzySets.add(FuzzySet.trapez("mittel", 28, 36, 78, 86));
        fuzzySets.add(FuzzySet.trapez("hoch", 78, 86, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable geschwindigkeitSzenario4() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 15, 30));
        fuzzySets.add(FuzzySet.trapez("mittel", 15, 30, 60, 80));
        fuzzySets.add(FuzzySet.trapez("hoch", 60, 80, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable geschwindigkeitSzenario5() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 20, 30));
        fuzzySets.add(FuzzySet.trapez("mittel", 20, 30, 70, 80));
        fuzzySets.add(FuzzySet.trapez("hoch", 70, 80, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable geschwindigkeitSzenario6() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 23, 33));
        fuzzySets.add(FuzzySet.trapez("mittel", 23, 33, 73, 83));
        fuzzySets.add(FuzzySet.trapez("hoch", 73, 83, 254, 254));
        return new LinguistischeVariable(VKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable anteilLkw() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("sehr gering", 0, 0, 3, 7));
        fuzzySets.add(FuzzySet.trapez("gering", 3, 7, 8, 12));
        fuzzySets.add(FuzzySet.trapez("mittel", 8, 12, 18, 22));
        fuzzySets.add(FuzzySet.trapez("hoch", 18, 22, 23, 27));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 23, 27, 100, 100));
        return new LinguistischeVariable(ALKW, new Wertebereich(0, 100), fuzzySets);
    }

    public static LinguistischeVariable verkehrsunruhe() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 18, 22));
        fuzzySets.add(FuzzySet.trapez("hoch", 18, 22, 254, 254));
        return new LinguistischeVariable(SKFZ, new Wertebereich(0, 254), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario1() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 25, 35));
        fuzzySets.add(FuzzySet.trapez("hoch", 25, 35, 50, 70));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 50, 70, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 50, 70));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario2() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 35, 45));
        fuzzySets.add(FuzzySet.trapez("hoch", 35, 45, 60, 80));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 60, 80, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 60, 80));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario3() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 35, 45));
        fuzzySets.add(FuzzySet.trapez("hoch", 35, 45, 60, 80));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 45, 55, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 45, 55));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario4() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 15, 25));
        fuzzySets.add(FuzzySet.trapez("hoch", 15, 25, 40, 60));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 40, 60, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 40, 60));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario5() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 35, 45));
        fuzzySets.add(FuzzySet.trapez("hoch", 35, 45, 60, 80));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 60, 80, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 60, 80));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable bemessungsdichteSzenario6() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("niedrig", 0, 0, 25, 35));
        fuzzySets.add(FuzzySet.trapez("hoch", 25, 35, 50, 70));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 50, 70, 10000, 10000));
        fuzzySets.add(FuzzySet.trapez("nicht sehr hoch", 0, 0, 50, 70));
        return new LinguistischeVariable(KB, new Wertebereich(0, 10000), fuzzySets);
    }

    public static LinguistischeVariable verkehrsstaerkeSzenario1() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("sehr niedrig", 0, 0, 500, 700));
        fuzzySets.add(FuzzySet.trapez("niedrig", 500, 700, 1000, 1400));
        fuzzySets.add(FuzzySet.trapez("mittel", 1000, 1400, 2000, 2400));
        fuzzySets.add(FuzzySet.trapez("hoch", 2000, 2400, 3000, 3400));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 3000, 3400, 100000000, 100000000));
        return new LinguistischeVariable(QKFZ, new Wertebereich(0, 100000000), fuzzySets);
    }

    public static LinguistischeVariable verkehrsstaerkeSzenario2() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("sehr niedrig", 0, 0, 750, 1050));
        fuzzySets.add(FuzzySet.trapez("niedrig", 750, 1050, 1500, 2100));
        fuzzySets.add(FuzzySet.trapez("mittel", 1500, 2100, 3000, 3600));
        fuzzySets.add(FuzzySet.trapez("hoch", 3000, 3600, 4500, 5100));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 4500, 5100, 100000000, 100000000));
        return new LinguistischeVariable(QKFZ, new Wertebereich(0, 100000000), fuzzySets);
    }

    public static LinguistischeVariable verkehrsstaerkeSzenario3() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("sehr niedrig", 0, 0, 1000, 1400));
        fuzzySets.add(FuzzySet.trapez("niedrig", 1000, 1400, 2000, 2800));
        fuzzySets.add(FuzzySet.trapez("mittel", 2000, 2800, 3000, 3600));
        fuzzySets.add(FuzzySet.trapez("hoch", 3000, 3600, 6000, 6800));
        fuzzySets.add(FuzzySet.trapez("sehr hoch", 6000, 6800, 100000000, 100000000));
        return new LinguistischeVariable(QKFZ, new Wertebereich(0, 100000000), fuzzySets);
    }

    public static Regelbasis regelbasis() {
        List<Regel> regeln = new ArrayList<>();
        fuegeRegelnFuerVerkehrszustandHinzu(regeln);
        fuegeRegelnFuerVerkehrszustandUnruheHinzu(regeln);
        return new Regelbasis(regeln);
    }

    private static void fuegeRegelnFuerVerkehrszustandHinzu(List<Regel> regeln) {
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(VKFZ, "niedrig"),
                        new TermVariable(VKFZ_1, "niedrig"))
                )).dann(new TermVariable(VERKEHRSZUSTAND, "gestaut")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Collections.singletonList(
                        new TermVariable(KB, "sehr hoch"))
                )).dann(new TermVariable(VERKEHRSZUSTAND, "gestaut")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(VKFZ, "mittel"),
                        new TermVariable(KB, "nicht sehr hoch"))
                )).dann(new TermVariable(VERKEHRSZUSTAND, "zähfließend")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(VKFZ, "hoch"),
                        new TermVariable(KB, "hoch"))
                )).dann(new TermVariable(VERKEHRSZUSTAND, "dicht")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(VKFZ, "hoch"),
                        new TermVariable(KB, "niedrig"))
                )).dann(new TermVariable(VERKEHRSZUSTAND, "frei")
                ).erzeuge()
        );
    }

    private static void fuegeRegelnFuerVerkehrszustandUnruheHinzu(List<Regel> regeln) {
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(SKFZ, "hoch"),
                        new TermVariable(QKFZ, "hoch"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "Unruhe im Verkehr")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(SKFZ, "hoch"),
                        new TermVariable(QKFZ, "sehr hoch"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "Unruhe im Verkehr")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(SKFZ, "hoch"),
                        new TermVariable(QKFZ, "mittel"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "keine Unruhe")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(SKFZ, "hoch"),
                        new TermVariable(QKFZ, "niedrig"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "keine Unruhe")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(SKFZ, "hoch"),
                        new TermVariable(QKFZ, "sehr niedrig"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "keine Unruhe")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Collections.singletonList(
                        new TermVariable(SKFZ, "niedrig"))
                )).dann(new TermVariable(VERKEHRSZUSTAND_UNRUHE, "keine Unruhe")
                ).erzeuge()
        );
    }

}
