/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;

import java.util.*;

class FuzzyWerteCache {

    private Map<Systemobjekt, Map<String, Deque<FuzzyVariable>>> cache = new LinkedHashMap<>();

    private final int maxZyklen;

    FuzzyWerteCache(int maxZyklen) {
        this.maxZyklen = maxZyklen;
    }

    void cacheFuzzyWerte(Collection<Datensatz<?, FuzzyVariable>> fuzzyWerte) {
        fuzzyWerte.forEach(datensatz -> {
            if (datensatz.getDatum().isNull()) {
                return;
            }

            Systemobjekt systemobjekt = datensatz.getSystemobjekt();
            if (!cache.containsKey(systemobjekt)) {
                cache.put(systemobjekt, new LinkedHashMap<>());
            }

            Map<String, Deque<FuzzyVariable>> fuzzyVariablen = cache.get(systemobjekt);
            if (!fuzzyVariablen.containsKey(datensatz.getDatum().getName())) {
                fuzzyVariablen.put(datensatz.getDatum().getName(), new LinkedList<>());
            }

            Deque<FuzzyVariable> zyklen = fuzzyVariablen.get(datensatz.getDatum().getName());
            zyklen.addFirst(datensatz.getDatum());
            while (zyklen.size() > maxZyklen) {
                zyklen.removeLast();
            }
        });
    }

    void setzeFuzzyWerteInKontextFuer(Systemobjekt systemobjekt, Map<String, Object> kontext) {
        if (!cache.containsKey(systemobjekt)) {
            return;
        }

        cache.get(systemobjekt).forEach((name, variablen) -> {
            int i = 0;
            for (FuzzyVariable variable : variablen) {
                if (i == 0) {
                    kontext.put(name, variable);
                } else {
                    kontext.put(name + i, variable);
                }
                i++;
            }
        });
    }

}
